@@ -1,7 +1,7 @@
 #!/bin/bash
-# Copyright 1999-2004 Gentoo Technologies, Inc.
+# Copyright 1999-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/env-update.sh,v 1.3 2004/04/21 17:09:18 vapier Exp $
+# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/env-update.sh,v 1.5 2004/10/18 17:35:45 vapier Exp $
 
 source /etc/init.d/functions.sh
 
@@ -23,6 +23,23 @@ note:
 
 export SVCDIR="${svcdir}"
 
+# Only update if files have actually changed
+update=1
+if [ "$1" == "-u" ]
+then
+	update=0
+	for config in /etc/env.d
+	do
+		if [ "${config}" -nt "${svcdir}/envcache" ]
+		then
+			update=1
+			break
+		fi
+	done
+	shift
+fi
+[ ${update} -eq 0 ] && exit 0
+
 if [ "$#" -ne 0 ]
 then
 	usage
