@@ -1,6 +1,6 @@
-# Copyright 1999-2004 Gentoo Technologies, Inc.
+# Copyright 1999-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/functions.sh,v 1.55 2004/08/18 20:35:37 agriffis Exp $
+# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/functions.sh,v 1.75 2004/10/31 11:27:38 azarah Exp $
 
 RC_GOT_FUNCTIONS="yes"
 
@@ -25,9 +25,12 @@ ordtypes="before after"
 
 # Dont output to stdout?
 RC_QUIET_STDOUT="no"
+RC_VERBOSE="${RC_VERBOSE:-no}"
 
 # Should we use color?
-RC_NOCOLOR="no"
+RC_NOCOLOR="${RC_NOCOLOR:-no}"
+# Can the terminal handle endcols?
+RC_ENDCOL="yes"
 
 #
 # Default values for rc system
@@ -37,12 +40,20 @@ RC_NET_STRICT_CHECKING="no"
 RC_PARALLEL_STARTUP="no"
 RC_USE_CONFIG_PROFILE="yes"
 
+# 
+# Default values for e-message indentation and dots
+#
+RC_INDENTATION=''
+RC_DEFAULT_INDENT=3
+#RC_DOT_PATTERN=' .'
+RC_DOT_PATTERN=''
+
 # Override defaults with user settings ...
 [ -f /etc/conf.d/rc ] && source /etc/conf.d/rc
 
 # void splash(...)
 #
-#  Notify bootsplash/splashutils/gensplash/whatever about 
+#  Notify bootsplash/splashutils/gensplash/whatever about
 #  important events.
 #
 splash() {
@@ -124,6 +135,20 @@ setup_defaultlevels() {
 	return 0
 }
 
+# void get_libdir(void)
+#
+#    prints the current libdir {lib,lib32,lib64}
+#
+get_libdir()
+{
+	if [ -n "${CONF_LIBDIR_OVERRIDE}" ]; then
+		CONF_LIBDIR="${CONF_LIBDIR_OVERRIDE}"
+	elif [ -x "/usr/bin/portageq" ]; then
+		CONF_LIBDIR="$(/usr/bin/portageq envvar CONF_LIBDIR)"
+	fi
+	echo ${CONF_LIBDIR:=lib}
+}
+
 # void esyslog(char* priority, char* tag, char* message)
 #
 #    use the system logger to log a message
@@ -138,24 +163,52 @@ esyslog() {
 		tag="$2"
 		
 		shift 2
-		[ -z "$*" ] && return 0
+		[ -z "$@" ] && return 0
 		
-		/usr/bin/logger -p "${pri}" -t "${tag}" -- "$*"
+		/usr/bin/logger -p "${pri}" -t "${tag}" -- "$@"
 	fi
 
 	return 0
 }
 
+# void eindent(int num)
+#
+#    increase the indent used for e-commands.
+#
+eindent() {
+	local i=$1
+	(( i > 0 )) || (( i = RC_DEFAULT_INDENT ))
+	esetdent $(( ${#RC_INDENTATION} + i ))
+}
+
+# void eoutdent(int num)
+#
+#    decrease the indent used for e-commands.
+#
+eoutdent() {
+	local i=$1
+	(( i > 0 )) || (( i = RC_DEFAULT_INDENT ))
+	esetdent $(( ${#RC_INDENTATION} - i ))
+}
+
+# void esetdent(int num)
+#
+#    hard set the indent used for e-commands.
+#    num defaults to 0
+#
+esetdent() {
+	local i=$1
+	(( i < 0 )) && (( i = 0 ))
+	RC_INDENTATION=$(printf "%${i}s" '')
+}
+
 # void einfo(char* message)
 #
 #    show an informative message (with a newline)
 #
 einfo() {
-	if [ "${RC_QUIET_STDOUT}" != "yes" ]
-	then
-		echo -e " ${GOOD}*${NORMAL} ${*}"
-	fi
-
+	einfon "$@\n"
+	LAST_E_CMD=einfo
 	return 0
 }
 
@@ -164,11 +217,10 @@ einfo() {
 #    show an informative message (without a newline)
 #
 einfon() {
-	if [ "${RC_QUIET_STDOUT}" != "yes" ]
-	then
-		echo -ne " ${GOOD}*${NORMAL} ${*}"
-	fi
-
+	[[ ${RC_QUIET_STDOUT} == yes ]] && return 0
+	[[ ${RC_ENDCOL} != yes && ${LAST_E_CMD} == ebegin ]] && echo
+	echo -ne " ${GOOD}*${NORMAL} ${RC_INDENTATION}$@"
+	LAST_E_CMD=einfon
 	return 0
 }
 
@@ -177,16 +229,17 @@ einfon() {
 #    show a warning message + log it
 #
 ewarn() {
-	if [ "${RC_QUIET_STDOUT}" = "yes" ]
-	then
-		echo " ${*}"
+	if [[ ${RC_QUIET_STDOUT} == yes ]]; then
+		echo " $@"
 	else
-		echo -e " ${WARN}*${NORMAL} ${*}"
+		[[ ${RC_ENDCOL} != yes && ${LAST_E_CMD} == ebegin ]] && echo
+		echo -e " ${WARN}*${NORMAL} ${RC_INDENTATION}$@"
 	fi
 
 	# Log warnings to system log
-	esyslog "daemon.warning" "rc-scripts" "${*}"
+	esyslog "daemon.warning" "rc-scripts" "$@"
 
+	LAST_E_CMD=ewarn
 	return 0
 }
 
@@ -195,16 +248,17 @@ ewarn() {
 #    show an error message + log it
 #
 eerror() {
-	if [ "${RC_QUIET_STDOUT}" = "yes" ]
-	then
-		echo " ${*}" >/dev/stderr
+	if [[ ${RC_QUIET_STDOUT} == yes ]]; then
+		echo " $@" >/dev/stderr
 	else
-		echo -e " ${BAD}*${NORMAL} ${*}"
+		[[ ${RC_ENDCOL} != yes && ${LAST_E_CMD} == ebegin ]] && echo
+		echo -e " ${BAD}*${NORMAL} ${RC_INDENTATION}$@"
 	fi
 
 	# Log errors to system log
-	esyslog "daemon.err" "rc-scripts" "${*}"
+	esyslog "daemon.err" "rc-scripts" "$@"
 
+	LAST_E_CMD=eerror
 	return 0
 }
 
@@ -213,17 +267,59 @@ eerror() {
 #    show a message indicating the start of a process
 #
 ebegin() {
-	if [ "${RC_QUIET_STDOUT}" != "yes" ]
-	then
-		if [ "${RC_NOCOLOR}" = "yes" ]
-		then
-			echo -ne " ${GOOD}*${NORMAL} ${*}..."
+	local msg="$@" dots spaces=${RC_DOT_PATTERN//?/ }
+	[[ ${RC_QUIET_STDOUT} == yes ]] && return 0
+
+	if [[ -n ${RC_DOT_PATTERN} ]]; then
+		dots=$(printf "%$(( COLS - 3 - ${#RC_INDENTATION} - ${#msg} - 7 ))s" '')
+		dots=${dots//${spaces}/${RC_DOT_PATTERN}}
+		msg="${msg}${dots}"
+	else
+		msg="${msg} ..."
+	fi
+	einfon "${msg}"
+	[[ ${RC_ENDCOL} == yes ]] && echo
+
+	LAST_E_LEN=$(( 3 + ${#RC_INDENTATION} + ${#msg} ))
+	LAST_E_CMD=ebegin
+	return 0
+}
+
+# void _eend(int error, char *efunc, char* errstr)
+#
+#    indicate the completion of process, called from eend/ewend
+#    if error, show errstr via efunc
+#
+#    This function is private to functions.sh.  Do not call it from a
+#    script.
+#
+_eend() {
+	local retval=${1:-0} efunc=${2:-eerror} msg
+	shift 2
+
+	if [[ ${retval} == 0 ]]; then
+		[[ ${RC_QUIET_STDOUT} == yes ]] && return 0
+		msg="${BRACKET}[ ${GOOD}ok${BRACKET} ]${NORMAL}"
+	else
+		if [[ -c /dev/null ]]; then
+			rc_splash "stop" &>/dev/null &
 		else
-			echo -e " ${GOOD}*${NORMAL} ${*}..."
+			rc_splash "stop" &
 		fi
+		if [[ -n "$@" ]]; then
+			${efunc} "$@"
+		fi
+		msg="${BRACKET}[ ${BAD}!!${BRACKET} ]${NORMAL}"
 	fi
 
-	return 0
+	if [[ ${RC_ENDCOL} == yes ]]; then
+		echo -e "${ENDCOL}  ${msg}"
+	else
+		[[ ${LAST_E_CMD} == ebegin ]] || LAST_E_LEN=0
+		printf "%$(( COLS - LAST_E_LEN - 6 ))s%b\n" '' "${msg}"
+	fi
+
+	return ${retval}
 }
 
 # void eend(int error, char* errstr)
@@ -232,71 +328,44 @@ ebegin() {
 #    if error, show errstr via eerror
 #
 eend() {
-	local retval=
-	
-	if [ "$#" -eq 0 ] || ([ -n "$1" ] && [ "$1" -eq 0 ])
-	then
-		if [ "${RC_QUIET_STDOUT}" != "yes" ]
-		then
-			echo -e "${ENDCOL}  ${BRACKET}[ ${GOOD}ok${BRACKET} ]${NORMAL}"
-		fi
-	else
-		retval="$1"
-		
-		if [ -c /dev/null ] ; then
-			rc_splash "stop" &>/dev/null &
-		else
-			rc_splash "stop" &
-		fi
-		
-		if [ "$#" -ge 2 ]
-		then
-			shift
-			eerror "${*}"
-		fi
-		if [ "${RC_QUIET_STDOUT}" != "yes" ]
-		then
-			echo -e "${ENDCOL}  ${BRACKET}[ ${BAD}!!${BRACKET} ]${NORMAL}"
-			# extra spacing makes it easier to read
-			echo
-		fi
-		return ${retval}
-	fi
+	local retval=${1:-0}
+	shift
 
-	return 0
+	_eend ${retval} eerror "$@"
+
+	LAST_E_CMD=eend
+	return $retval
 }
 
-# void ewend(int error, char *warnstr)
+# void ewend(int error, char* errstr)
 #
 #    indicate the completion of process
-#    if error, show warnstr via ewarn
+#    if error, show errstr via ewarn
 #
 ewend() {
-	local retval=
-	
-	if [ "$#" -eq 0 ] || ([ -n "$1" ] && [ "$1" -eq 0 ])
-	then
-		if [ "${RC_QUIET_STDOUT}" != "yes" ]
-		then
-			echo -e "${ENDCOL}  ${BRACKET}[ ${GOOD}ok${BRACKET} ]${NORMAL}"
-		fi
-	else
-		retval="$1"
-		if [ "$#" -ge 2 ]
-		then
-			shift
-			ewarn "${*}"
-		fi
-		if [ "${RC_QUIET_STDOUT}" != "yes" ]
-		then
-			echo -e "${ENDCOL}  ${BRACKET}[ ${WARN}!!${BRACKET} ]${NORMAL}"
-			# extra spacing makes it easier to read
-			echo
-		fi
-		return "${retval}"
-	fi
+	local retval=${1:-0}
+	shift
 
-	return 0
+	_eend ${retval} ewarn "$@"
+
+	LAST_E_CMD=ewend
+	return $retval
+}
+
+# v-e-commands honor RC_VERBOSE which defaults to no.
+# The condition is negated so the return value will be zero.
+veinfo() { [[ "${RC_VERBOSE}" != yes ]] || einfo "$@"; }
+veinfon() { [[ "${RC_VERBOSE}" != yes ]] || einfon "$@"; }
+vewarn() { [[ "${RC_VERBOSE}" != yes ]] || ewarn "$@"; }
+veerror() { [[ "${RC_VERBOSE}" != yes ]] || eerror "$@"; }
+vebegin() { [[ "${RC_VERBOSE}" != yes ]] || ebegin "$@"; }
+veend() { 
+	[[ "${RC_VERBOSE}" == yes ]] && { eend "$@"; return $?; }
+	return ${1:-0}
+}
+veend() { 
+	[[ "${RC_VERBOSE}" == yes ]] && { ewend "$@"; return $?; }
+	return ${1:-0}
 }
 
 # bool wrap_rcscript(full_path_and_name_of_rc-script)
@@ -380,30 +449,30 @@ KV_to_int() {
 	local KV_MINOR=
 	local KV_MICRO=
 	local KV_int=
-	
+
 	[ -z "$1" ] && return 1
-    
+
 	KV_MAJOR="$(KV_major "$1")"
 	KV_MINOR="$(KV_minor "$1")"
 	KV_MICRO="$(KV_micro "$1")"
-	KV_int="$((KV_MAJOR * 65536 + KV_MINOR * 256 + KV_MICRO))"
-    
+	KV_int="$(( KV_MAJOR * 65536 + KV_MINOR * 256 + KV_MICRO ))"
+
 	# We make version 2.2.0 the minimum version we will handle as
 	# a sanity check ... if its less, we fail ...
 	if [ "${KV_int}" -ge 131584 ]
-	then 
+	then
 		echo "${KV_int}"
 
 		return 0
 	fi
 
 	return 1
-}   
+}
 
 # int get_KV()
 #
 #    return the kernel version (major, minor and micro concated) as an integer
-#   
+#
 get_KV() {
 	local KV="$(uname -r)"
 
@@ -421,14 +490,14 @@ get_KV() {
 get_bootparam() {
 	local x copt params retval=1
 
-	[ ! -e "/proc/cmdline" ] && return 1
+	[ ! -r "/proc/cmdline" ] && return 1
 	
 	for copt in $(< /proc/cmdline)
 	do
 		if [ "${copt%=*}" = "gentoo" ]
 		then
 			params="$(gawk -v PARAMS="${copt##*=}" '
-				BEGIN { 
+				BEGIN {
 					split(PARAMS, nodes, ",")
 					for (x in nodes)
 						print nodes[x]
@@ -464,7 +533,7 @@ dolisting() {
 	local y=
 	local tmpstr=
 	local mylist=
-	local mypath="${*}"
+	local mypath="$@"
 
 	if [ "${mypath%/\*}" != "${mypath}" ]
 	then
@@ -506,7 +575,7 @@ save_options() {
 		mkdir -p -m 0755 "${svcdir}/options/${myservice}"
 	fi
 	
-	echo "$*" > "${svcdir}/options/${myservice}/${myopts}"
+	echo "$@" > "${svcdir}/options/${myservice}/${myopts}"
 
 	return 0
 }
@@ -540,10 +609,43 @@ add_suffix() {
 	return 0
 }
 
-getcols() {
-	echo "$2"
+# Network filesystems list for common use in rc-scripts.
+# This variable is used in is_net_fs and other places such as
+# localmount.
+NET_FS_LIST="afs cifs coda ncpfs nfs nfs4 shfs smbfs"
+
+# bool is_net_fs(path)
+#
+#   return 0 if path is the mountpoint of a networked filesystem
+#
+#   EXAMPLE:  if is_net_fs / ; then ...
+#
+is_net_fs() {
+	local fstype=$(mount -o remount -fv "$1" | awk '{print $(NF-1)}')
+	[[ " ${NET_FS_LIST} " == *" ${fstype} "* ]]
+	return $?
 }
 
+# bool is_uml_sys()
+#
+#   return 0 if the currently running system is User Mode Linux
+#
+#   EXAMPLE:  if is_uml_sys ; then ...
+#
+is_uml_sys() {
+	grep -q 'UML' /proc/cpuinfo &> /dev/null
+	return $?
+}
+
+
+##############################################################################
+#                                                                            #
+# This should be the last code in here, please add all functions above!!     #
+#                                                                            #
+# *** START LAST CODE ***                                                    #
+#                                                                            #
+##############################################################################
+
 if [ -z "${EBUILD}" ]
 then
 	# Setup a basic $PATH.  Just add system default to existing.
@@ -553,62 +655,52 @@ then
 
 	if [ "$(/sbin/consoletype 2> /dev/null)" = "serial" ]
 	then
-		# We do not want colors on serial terminals
+		# We do not want colors/endcols on serial terminals
 		RC_NOCOLOR="yes"
+		RC_ENDCOL="no"
 	fi
 	
-	for arg in $*
+	for arg in "$@"
 	do
 		case "${arg}" in
 			# Lastly check if the user disabled it with --nocolor argument
-			--nocolor)
+			--nocolor|-nc)
 				RC_NOCOLOR="yes"
 				;;
 		esac
 	done
 
-	if [ -e "/proc/cmdline" ]
+	if [ -r "/proc/cmdline" ]
 	then
 		setup_defaultlevels
 	fi
 else
 	# Should we use colors ?
-	if [ "${*/depend}" = "$*" ]
-	then
+	if [[ $@ != *depend* ]]; then
 		# Check user pref in portage
 		RC_NOCOLOR="$(portageq envvar NOCOLOR 2>/dev/null)"
-		
 		[ "${RC_NOCOLOR}" = "true" ] && RC_NOCOLOR="yes"
 	else
 		# We do not want colors or stty to run during emerge depend
 		RC_NOCOLOR="yes"
-	fi                                                                                                                       
+		RC_ENDCOL="no"
+	fi
 fi
 
-if [ "${RC_NOCOLOR}" = "yes" ]
-then
-	COLS="25 80"
-	ENDCOL=
-	
-	GOOD=
-	WARN=
-	BAD=
-	NORMAL=
-	HILITE=
-	BRACKET=
-	
-	if [ -n "${EBUILD}" ] && [ "${*/depend}" = "$*" ]
-	then
-		stty cols 80 &>/dev/null
-		stty rows 25 &>/dev/null
-	fi
+# Setup COLS and ENDCOL so eend can line up the [ ok ]
+COLS=${COLUMNS:-0}		# bash's internal COLUMNS variable
+(( COLS == 0 )) && COLS=$(stty size 2>/dev/null | cut -d' ' -f2)
+(( COLS > 0 )) || (( COLS = 80 ))	# width of [ ok ] == 7
+if [[ ${RC_ENDCOL} == yes ]]; then
+	ENDCOL=$'\e[A\e['$(( COLS - 7 ))'G'
+else
+	ENDCOL=''
+fi
+
+# Setup the colors so our messages all look pretty
+if [[ ${RC_NOCOLOR} == yes ]]; then
+	unset GOOD WARN BAD NORMAL HILITE BRACKET
 else
-	COLS="`stty size 2> /dev/null`"
-	COLS="`getcols ${COLS}`"
-	COLS=$((${COLS} - 7))
-	ENDCOL=$'\e[A\e['${COLS}'G'    # Now, ${ENDCOL} will move us to the end of the
-	                               # column;  irregardless of character width
-	
 	GOOD=$'\e[32;01m'
 	WARN=$'\e[33;01m'
 	BAD=$'\e[31;01m'
@@ -617,5 +709,13 @@ else
 	BRACKET=$'\e[34;01m'
 fi
 
+##############################################################################
+#                                                                            #
+# *** END LAST CODE ***                                                      #
+#                                                                            #
+# This should be the last code in here, please add all functions above!!     #
+#                                                                            #
+##############################################################################
+
 
 # vim:ts=4
