@@ -1,7 +1,7 @@
 #!/bin/bash
-# Copyright 1999-2004 Gentoo Technologies, Inc.
+# Copyright 1999-2004 Gentoo Foundation
 # Distributed under the terms of the GNU General Public License v2
-# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/runscript.sh,v 1.36 2004/08/18 20:35:37 agriffis Exp $
+# $Header: /var/cvsroot/gentoo-src/rc-scripts/sbin/runscript.sh,v 1.40 2004/10/31 12:26:32 azarah Exp $
 
 # Common functions
 [ "${RC_GOT_FUNCTIONS}" != "yes" ] && source /sbin/functions.sh
@@ -11,7 +11,7 @@
 [ "${RC_GOT_DAEMON}" != "yes" ] && source "${svclib}/sh/rc-daemon.sh"
 
 # Fix bug 48595
-if [[ $(id -u) != 0 ]]; then
+if [[ ${EUID} != 0 ]] ; then
 	eerror "ERROR: must be root to run init scripts"
 	exit 1
 fi
@@ -131,12 +131,9 @@ svc_stop() {
 			if in_runlevel "${myservice}" "${BOOTLEVEL}" || \
 			   in_runlevel "${myservice}" "${mylevel}"
 			then
-				local netcount="$(ls -1 "${svcdir}"/started/net.* 2> /dev/null | \
-					grep -v 'net\.lo' | egrep -c "\/net\.[[:alnum:]]+$")"
-
 				# Only worry about net.* services if this is the last one running,
 				# or if RC_NET_STRICT_CHECKING is set ...
-				if [ "${netcount}" -lt 1 -o "${RC_NET_STRICT_CHECKING}" = "yes" ]
+				if is_net_up
 				then
 					mydeps="net"
 				fi
@@ -276,13 +273,10 @@ svc_start() {
 						# A 'need' dependency is critical for startup
 						if [ "$?" -ne 0 ] && ineed -t "${myservice}" "${x}" >/dev/null
 						then
-							local netcount="$(ls -1 ${svcdir}/started/net.* 2> /dev/null | \
-								grep -v 'net\.lo' | egrep -c "\/net\.[[:alnum:]]+$")"
-						
 							# Only worry about a net.* service if we do not have one
-							# up and running already, or if RC_NET_SCTRICT_CHECKING
+							# up and running already, or if RC_NET_STRICT_CHECKING
 							# is set ....
-							if [ "${netcount}" -lt 1 -o "${RC_NET_STRICT_CHECKING}" = "yes" ]
+							if is_net_up
 							then
 								startfail="yes"
 							fi
@@ -447,6 +441,9 @@ do
 #	--nocolor)
 #		RC_NOCOLOR="yes"
 #		;;
+	--verbose)
+		RC_VERBOSE="yes"
+		;;
 	esac
 done
 for arg in $*
